import os
from io import StringIO
from tempfile import TemporaryDirectory, NamedTemporaryFile
from unittest import TestCase
from unittest.mock import patch

from textwrap import dedent

from rayter_generator.main import _main

class TestGenerator(TestCase):
    def setUp(self):
        super().setUp()
        self.games_path = TemporaryDirectory()
        self.output_path = TemporaryDirectory()

        with open(os.path.join(self.games_path.name, "scrabble.txt"), "w") as f:
            f.write(dedent("""
                # Scrabble
                game_name Scrabble

                game 2023-02-20 17:06
                PlayerOne   350
                PlayerTwo   320

                game 2023-02-24 13:00
                PlayerOne   346
                PlayerTwo   390
            """))

        with open(os.path.join(self.output_path.name, "players_metadata.json"), "w") as f:
            f.write(dedent("""
                {
                    "PlayerOne": {
                        "displayName": "Player One DisplayName"
                    }
                }
            """))

    def tearDown(self):
        self.games_path.cleanup()
        self.output_path.cleanup()
        super().tearDown()

    @patch("sys.stderr", new_callable=StringIO)
    @patch("sys.stdout", new_callable=StringIO)
    def test_generate_website(self, mock_stdout, mock_stderr):
        _main([
            "--games-path", self.games_path.name,
            "--output", self.output_path.name,
        ])
        self.assertTrue(os.path.exists(os.path.join(self.output_path.name, "index.html")))
        self.assertTrue(os.path.exists(os.path.join(self.output_path.name, "scrabble", "index.html")))

        with open(os.path.join(self.output_path.name, "index.html"), "r") as f:
            contents = f.read()
        self.assertIn("Scrabble", contents)

        with open(os.path.join(self.output_path.name, "scrabble", "index.html"), "r") as f:
            contents = f.read()
        self.assertIn("PlayerOne", contents)

        mock_stdout.seek(0)
        mock_stderr.seek(0)
        self.assertIn("scrabble/index.html", mock_stdout.read())

    @patch("sys.stderr", new_callable=StringIO)
    @patch("sys.stdout", new_callable=StringIO)
    def test_generate_website_with_config(self, mock_stdout, mock_stderr):
        with NamedTemporaryFile() as config_file:
            config_file.write(dedent("""
                site_name = "TestName"
            """).encode("utf-8"))
            config_file.flush()

            _main([
                "--games-path", self.games_path.name,
                "--output", self.output_path.name,
                "--config", config_file.name,
            ])
            with open(os.path.join(self.output_path.name, "index.html"), "r") as config_file:
                contents = config_file.read()
            self.assertIn("TestName", contents)

    @patch("sys.stderr", new_callable=StringIO)
    @patch("sys.stdout", new_callable=StringIO)
    def test_ignore_files_with_no_games(self, mock_stdout, mock_stderr):
        with open(os.path.join(self.games_path.name, "sometext.txt"), "w") as text_file:
            text_file.write(dedent("""
                This is just some text in a file.
            """))
        with open(os.path.join(self.games_path.name, "epmty.txt"), "w") as empty_file:
            empty_file.write(dedent(""))

        # Should not raise an exception
        _main([
            "--games-path", self.games_path.name,
            "--output", self.output_path.name,
        ])

        with open(os.path.join(self.output_path.name, "index.html"), "r") as f:
            contents = f.read()
        self.assertNotIn("sometext", contents)
        self.assertNotIn("empty", contents)

    @patch("sys.stderr", new_callable=StringIO)
    @patch("sys.stdout", new_callable=StringIO)
    def test_generate_player_pages(self, mock_stdout, mock_stderr):
        _main([
            "--games-path", self.games_path.name,
            "--output", self.output_path.name,
        ])

        with open(os.path.join(self.output_path.name, "player", "PlayerOne", "index.html"), "r") as f:
            contents = f.read()
        self.assertIn("Scrabble", contents)

    @patch("sys.stderr", new_callable=StringIO)
    @patch("sys.stdout", new_callable=StringIO)
    def test_player_metadata(self, mock_stdout, mock_stderr):
        _main([
            "--games-path", self.games_path.name,
            "--output", self.output_path.name,
            "--players", os.path.join(self.output_path.name, "players_metadata.json"),
        ])

        with open(os.path.join(self.output_path.name, "player", "PlayerOne", "index.html"), "r") as f:
            contents = f.read()
        self.assertIn("Player One DisplayName", contents)



import argparse
import os
import pathlib
import shutil
import sys

from .env import GeneratorEnvironment
from .games import parse_game_file
from .players import get_players
from .render import render_game_page, render_index_page, render_player_page
from .settings import ROOT_DIR


def _main(args):
    arg_parser = argparse.ArgumentParser(
        description="Generate Rayter website from game files",
    )
    arg_parser.add_argument(
        "--games-path",
        type=pathlib.Path,
        help="Path to directory containing game files",
        required=True,
    )
    arg_parser.add_argument(
        "--output",
        type=pathlib.Path,
        help="Path to directory to output rendered files",
        required=True,
    )
    arg_parser.add_argument(
        "--config",
        type=pathlib.Path,
        help="Path to configuration file in TOML format. If not specified, will look for config file at: `./config/rayter.toml` and `./rayter.toml`",
        required=False,
    )
    arg_parser.add_argument(
        "--players",
        type=pathlib.Path,
        help="Path to player metadata file in JSON format. If not specified, will look for players file at: `./config/players.json` and `./players.json`",
        required=False,
    )
    args = arg_parser.parse_args(args)
    #print(args)

    try:
        env = GeneratorEnvironment(
            games_path=args.games_path,
            output_path=args.output,
            config_path=args.config,
            players_path=args.players,
        )

        games = []
        # Create game list from game files
        for filename in os.listdir(env.games_path):
            if not filename.endswith(".txt"):
                continue
            if os.path.isdir(filename):
                continue

            # remove .txt from filename
            slug = filename[:-4]
            game = parse_game_file(os.path.join(env.games_path, filename), slug)
            if game is not None:
                games.append(game)

        # render game pages
        for game in games:
            render_game_page(env, game)

        # Get players
        players = get_players(games, env)

        # render player pages
        for player in players.values():
            render_player_page(env, player)

        # render index page
        render_index_page(env, games)

        # recursively copy static files to output directory
        print("Copying static files")
        shutil.copytree(
            os.path.join(env.root_path, "static"),
            os.path.join(env.output_path, "static"),
            dirs_exist_ok=True,
        )
    except FileNotFoundError as e:
        sys.stderr.write(f"Error: {e.strerror} ({e.filename})\n")
        sys.exit(1)


def main():
    _main(sys.argv[1:])


if __name__ == '__main__':
    main()



import os

from jinja2 import Environment, FileSystemLoader

from .settings import ROOT_DIR


def render_game_page(env, game):
    filename = os.path.join(env.output_path, game['slug'], 'index.html')
    env.render_template("game.html", {
        "name": game["game_name"],
        "ratings": game["ratings"],
        "game_name": game["game_name"],
        "slug": game["slug"],
        "players": [], # FIXME Fix players list
        "STATIC_URL": "../static/",
    }, output_path=filename)

def render_player_page(env, player):
    filename = os.path.join(env.output_path, 'player', player['slug'], 'index.html')
    env.render_template("player.html", {
        "player": player,
        "STATIC_URL": "../../static/",
    }, output_path=filename)

def render_index_page(env, games):
    games.sort(key=lambda g: g["count"], reverse=True)
    filename = os.path.join(env.output_path, 'index.html')
    env.render_template("index.html", {
        "games": games,
        "players": [],
        "global_chart": [],
        "log": [],
        "STATIC_URL": "static/",
    }, output_path=filename)
